// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI6C500K/24DSI6C500K_1.x.x.x_GSC_DN/driver/main.h $
// $Rev: 54804 $
// $Date: 2024-07-08 08:07:15 -0500 (Mon, 08 Jul 2024) $

// 24DSI6C500K: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "24dsi6c500k.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_READ
#define	DEV_SUPPORTS_WAIT
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1

typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"24DSI6C500K"	// Upper case form of the below.
#define	DEV_NAME							"24dsi6c500k"	// MUST AGREE WITH XXXX_BASE_NAME

#define	DEV_VERSION							"2.4"			// FOR DEVICE SPECIFIC CODE ONLY!
// 2.4	Updated the make file for consistency.
//		Updated I/O code for consistency.
//		Updated the sources script for consistency.
//		Updated the start script script for consistency.
//		Updated for Fedora 38.
//		Renamed all AUTO_CALIBRATE content to AUTOCAL
//		Renamed all AUTO_CAL content to AUTOCAL
// 2.3	Updated to support the 6.x series kernel.
//		Enhanced the Analog Input Buffer Clear IOCTL service.
//		Updated comments for consistency.
//		Updated FALLTHROUGH processing.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
//		Updated per PLX code segregation.
//		Updated use of some macros.
// 2.2	Removed compiler warnings for Fedora 31.
//		Standardized various IOCTL system log messages.
//		Standardized processing of the initialization IOCTL service.
//		Standardized processing of the autocalibration IOCTL service.
// 2.1	Updated initial file content for consistency.
//		White space cleanup.
//		Corrected some data type and macro names.
//		Added support for the 5.x kernel series.
//		Updated per changes to common I/O code.
//		Renamed various DMA macro.
//		Updated for more modular DMA and interrupt code.
//		Modified so open succeeds even if initialization fails.
//		Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Modified code so open succeeds even if initialization fails.
// 2.0	Updated to use the newer common driver sources.
//		Updated the dev_read_startup() service.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		Updated gsc_dma_open() and gsc_dma_close().
//		The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Added an infinite I/O timeout option.
//		Corrected the IOCTL code for the register write and mod services.
// 1.1	Added the Sync Source query feature and IOCTL service.
// 1.0	Initial release.

// I/O services
#define	DEV_IO_STREAM_QTY					(DEV_IO_RX_STREAM_QTY + DEV_IO_TX_STREAM_QTY)
#define	DEV_IO_RX_STREAM_QTY				1
#define	DEV_IO_TX_STREAM_QTY				0

#define	GSC_READ_PIO_WORK_32_BIT

// WAIT services
#define	DEV_WAIT_GSC_ALL					DSI6C500K_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					DSI6C500K_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						DSI6C500K_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	s32					io_mode;			// PIO, DMA, DMDMA
	s32					overflow_check;		// Check overflow when performing I/O
	s32					pio_threshold;		// Use PIO if samples <= this.
	s32					timeout_s;			// I/O timeout in seconds.
	s32					underflow_check;	// Check underflow when performing I/O

	// Initialized by I/O service at time of use.

	int					abort;				// Also set via IOCTL service.
	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.
	int					non_blocking;		// Is this non-blocking I/O?

	// Initialized by device specific I/O create code.

	int					bytes_per_sample;	// Sample size in bytes.
	u32					io_reg_offset;		// Offset of device's I/O FIFO.
	VADDR_T				io_reg_vaddr;		// Address of device's I/O FIFO.

	void				(*dev_io_sw_init)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_close)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	void				(*dev_io_open)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	int					(*dev_io_startup)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io);
	long				(*dev_pio_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_bmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_dmdma_available)	(GSC_ALT_STRUCT_T* alt, dev_io_t* io, size_t count);
	long				(*dev_pio_xfer)			(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_bmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);
	long				(*dev_dmdma_xfer)		(GSC_ALT_STRUCT_T* alt, dev_io_t* io, const os_mem_t* mem, size_t count, os_time_tick_t st_end);

	struct
	{
		u32				abort;		// ...WAIT_IO_XXX_ABORT
		u32				done;		// ...WAIT_IO_XXX_DONE
		u32				error;		// ...WAIT_IO_XXX_ERROR
		u32				timeout;	// ...WAIT_IO_XXX_TIMEOUT
	} wait;

	// Initialize by GSC commone create code.

	os_mem_t			mem;				// I/O buffer.
	os_sem_t			sem;				// Only one Tx or Rx at a time.
};

struct _dev_data_t
{
	// Put an integral data type at beginning of structure.
	int					board_index;	// Index of the device being accessed.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					share;			// Were we opened in shared mode?
	int					users;			// Number of currently active open() requests.

	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For DMA based I/O.
	gsc_irq_t			irq;			// For interrupts.
	gsc_wait_node_t*	wait_list;

	struct
	{					// This is for streaming I/O
		dev_io_t		rx;				// For read operations.
		dev_io_t*		io_streams[DEV_IO_STREAM_QTY];
	} io;

	struct
	{
		VADDR_T			plx_intcsr_32;	// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;	// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;	// DMA Threshold Register

		VADDR_T			gsc_bctlr_32;	// 0x00 Board Control Register
		VADDR_T			gsc_rcr_32;		// 0x04 Rate Control Register
		VADDR_T			gsc_diopr_32;	// 0x08 Digital I/O Port Register
		VADDR_T			gsc_csar_32;	// 0x0C Clock Source Assignment Register
		VADDR_T			gsc_rdr_32;		// 0x10 Rate Divisors Register
										// 0x14 Reserved
										// 0x18 Reserved
		VADDR_T			gsc_bbsr_32;	// 0x1C Burst Block Size Register
		VADDR_T			gsc_bufcr_32;	// 0x20 Buffer Control Register
		VADDR_T			gsc_bcfgr_32;	// 0x24 Board Configuration Register
		VADDR_T			gsc_bufsr_32;	// 0x28 Buffer Size Register
		VADDR_T			gsc_avr_32;		// 0x2C Autocal Values Register
		VADDR_T			gsc_idbr_32;	// 0x30 Input Data Buffer Register
		VADDR_T			gsc_bttr_32;	// 0x3C Burst Trigger Timer Register

	} vaddr;

	struct
	{
		u32				gsc_bcfgr_32;		// Board Configuration Register

		s32				autocal_ms;			// Maximum ms for autocalibrate

		s32				channels_max;		// Maximum channels supported by model.
		s32				channel_qty;		// The number of channels on the device.

		s32				fclk_default;		// Burst Fclk default (Hz)
		s32				fifo_size;			// Size of FIFO - not the fill level.
		s32				filter_freq;		// 0, 2,000,000, -1, ...
		s32				fref_default;		// Sampling Fref default (Hz)
		s32				fsamp_max;			// The maximum Fsamp rate per channel.
		s32				fsamp_min;			// The minimum Fsamp rate per channel.

		s32				initialize_ms;		// Maximum ms for initialize

		s32				ndiv_max;			// Ndiv maximum
		s32				ndiv_min;			// Ndiv minimum
		s32				nref_max;			// Nref maximum
		s32				nref_min;			// Nref minimum
		s32				nvco_max;			// Nvco maximum
		s32				nvco_min;			// Nvco minimum

		s32				fgen_max;	// Rate Generator maximum output rate.
		s32				fgen_min;	// Rate Generator minimum output rate.

		s32				sync_src;			// External vs Group-0 Clock Assignment

		s32				voltage_range;		// 10/6/5/2.5
	} cache;
};



// prototypes *****************************************************************

void		dev_io_close(dev_data_t* dev);
int			dev_io_create(dev_data_t* dev);
void		dev_io_destroy(dev_data_t* dev);
int			dev_io_open(dev_data_t* dev);
dev_io_t*	dev_io_read_select(dev_data_t* dev, size_t bytes);
int			dev_irq_create(dev_data_t* dev);
void		dev_irq_destroy(dev_data_t* dev);
int			dev_read_create(dev_data_t* dev, dev_io_t* io);

int			initialize_ioctl(dev_data_t* dev, void* arg);



#endif
