// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI6C500K/24DSI6C500K_1.x.x.x_GSC_DN/samples/id/perform.c $
// $Rev: 54795 $
// $Date: 2024-07-08 07:52:54 -0500 (Mon, 08 Jul 2024) $

// 24DSI6C500K: Sample Application: source file

#include "main.h"



//*****************************************************************************
static void _input_modes(void)
{
	gsc_label("Input Modes");
	printf("Differential, Zero Test, +Vref Test\n");
}



//*****************************************************************************
static void _sampling_modes(void)
{
	gsc_label("Sampling Modes");
	printf("Continuous, Burst, Externally Clocked, SW Triggered\n");
}



//*****************************************************************************
static void _burst_modes(void)
{
	gsc_label("Burst Modes");
	printf("Continuous, Externally Triggered, SW Triggered\n");
}



//*****************************************************************************
static void _burst_block_size_range(void)
{
	gsc_label("Burst Block Size Range");
	printf("1 to 0xFFFFFF samples\n");
}



//*****************************************************************************
static void _bbt_divisor_range(void)
{
	gsc_label("Burst Timer Divisor Range");
	printf("0 to 0xFFFFFF\n");
}



//*****************************************************************************
static void _xcvr_modes(void)
{
	gsc_label("Transceiver Modes");
	printf("TTL, LVDS\n");
}



//*****************************************************************************
static void _dio_ports(void)
{
	gsc_label("Digital I/O Ports");
	printf("8-bits, Bidirectional Nibbles\n");
}



//*****************************************************************************
static int _id_device_reg(const args_t* args)
{
	int						errs;
	gsc_reg_def_t			list[2]	= { { NULL }, { NULL } };
	const gsc_reg_def_t*	ptr;

	printf("\n");
	ptr	= dsi6c500k_reg_get_def_id(DSI6C500K_GSC_BCFGR);

	if (ptr)
	{
		list[0]	= ptr[0];
		errs	= gsc_reg_list(args->fd, list, 1, dsi6c500k_reg_read);
	}
	else
	{
		errs	= 1;
		printf(	"FAIL <---  (%d. dsi6c500k_reg_get_def_id, DSI6C500K_GSC_BCFGR)\n",
				__LINE__);
	}

	return(errs);
}



//*****************************************************************************
static int _register_map(const args_t* args)
{
	int	errs;

	printf("\n");
	errs	= dsi6c500k_reg_list(args->fd, args->detail);
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;
	int	i;

	gsc_label("Device Features");
	printf("\n");

	gsc_label_level_inc();

	_input_modes();
	_sampling_modes();
	_burst_modes();
	_burst_block_size_range();
	_bbt_divisor_range();
	_xcvr_modes();
	_dio_ports();

	for (i = 0; i < DSI6C500K_QUERY_LAST; i++)
		errs	+= dsi6c500k_query(args->fd, -1, 1, i, NULL);

	gsc_label_level_dec();

	errs	+= _id_device_reg(args);
	errs	+= _register_map(args);
	printf("\n");

	return(errs);
}


