// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI6C500K/24DSI6C500K_1.x.x.x_GSC_DN/samples/rxrate/perform.c $
// $Rev: 52724 $
// $Date: 2023-04-03 14:02:52 -0500 (Mon, 03 Apr 2023) $

// 24DSI6C500K: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M	(1024L * 1024L)



// variables ******************************************************************

static u32	_rx[_1M];



//*****************************************************************************
static int _rate_test(const args_t* args, int seconds, long get)
{
	os_time_ns_t	begin;
	os_time_ns_t	end;
	int				errs		= 0;
	long long		data_limit	= args->rx_mb * 1000000;
	long long		data_total	= 0;
	long			got;
	long long		ms;
	long long		ms_limit	= 1000 * seconds;
	long long		ms_total	= 0;
	double			rate;
	long long		total		= 0;

	gsc_label("Reading");
	os_time_get_ns(&begin);

	for (;;)
	{
		got	= dsi6c500k_read(args->fd, _rx, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <--- (dsi6c500k_read");
			printf(", total so far ");
			gsc_label_long_comma(data_total);
			printf(", requested ");
			gsc_label_long_comma(get);
			printf(" bytes, got ");
			gsc_label_long_comma(got);
			printf(")\n");
			break;
		}

		os_time_get_ns(&end);
		data_total	+= got;
		ms_total	= ((long long) end.tv_sec * 1000 + end.tv_nsec / 1000000)
					- ((long long) begin.tv_sec * 1000 + begin.tv_nsec / 1000000);

		if ((ms_total >= ms_limit) && (data_total >= data_limit))
			break;
	}

	if (errs == 0)
	{
		printf(	"PASS  (");

		// Samples
		total	= data_total;
		gsc_label_long_comma(total / 4);
		printf(" Samples, ");

		// Period
		ms	= ms_total;
		gsc_label_float_comma((double) ms / 1000, -1, 3);
		printf(" Seconds, ");

		// Rate
		ms		= (ms <= 0) ? 1 : ms;
		rate	= ((double) total * 1000000 / 4) / ms / 1000;
		gsc_label_float_comma(rate, -1, 3);
		printf(" S/S)\n");
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_rx);
	int		seconds	= 5;

	errs	+= dsi6c500k_config_ai			(args->fd, -1, 1, -1, 500000L);
	errs	+= dsi6c500k_rx_io_mode			(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= dsi6c500k_rx_io_overflow		(args->fd, -1, 1, DSI6C500K_IO_OVERFLOW_IGNORE, NULL);
	errs	+= dsi6c500k_rx_io_underflow	(args->fd, -1, 1, DSI6C500K_IO_UNDERFLOW_IGNORE, NULL);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %ld MB minimum)\n", seconds, (long) args->rx_mb);
	gsc_label_level_inc();

	errs	+= dsi6c500k_ai_buf_clear		(args->fd, -1, 1);
	errs	+= dsi6c500k_ai_buf_overflow	(args->fd, -1, 1, DSI6C500K_AI_BUF_OVERFLOW_CLEAR, NULL);
	errs	+= dsi6c500k_ai_buf_underflow	(args->fd, -1, 1, DSI6C500K_AI_BUF_UNDERFLOW_CLEAR, NULL);

	errs	+= _rate_test(args, seconds, get);

	errs	+= dsi6c500k_ai_buf_overflow	(args->fd, -1, 1, -1, NULL);
	errs	+= dsi6c500k_ai_buf_underflow	(args->fd, -1, 1, -1, NULL);

	gsc_label_level_dec();
	return(errs);
}



