// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI6C500K/24DSI6C500K_1.x.x.x_GSC_DN/utils/util_channels_ready.c $
// $Rev: 46781 $
// $Date: 2020-01-09 17:04:13 -0600 (Thu, 09 Jan 2020) $

// 24DSI6C500K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi6c500k_channels_ready
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI6C500K_IOCTL_CHANNELS_READY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi6c500k_channels_ready(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Channels Ready", index);

	ret	= dsi6c500k_ioctl(fd, DSI6C500K_IOCTL_CHANNELS_READY, (void*) &set);

	if (ret)
	{
		if (ret == -ETIMEDOUT)
		{
			set	= DSI6C500K_CHANNELS_READY_NO;
		}
		else
		{
			errs++;
			printf("ERROR: dsi6c500k_ioctl() failure, error = %d\n", ret);
		}
	}

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DSI6C500K_CHANNELS_READY_NO:

			ptr	= "No, timed out";
			break;

		case DSI6C500K_CHANNELS_READY_YES:

			ptr	= "Yes";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}



