// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI6C500K/24DSI6C500K_1.x.x.x_GSC_DN/utils/util_irq_sel.c $
// $Rev: 54781 $
// $Date: 2024-07-08 07:40:17 -0500 (Mon, 08 Jul 2024) $

// 24DSI6C500K: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi6c500k_irq_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI6C500K_IOCTL_IRQ_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi6c500k_irq_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("IRQ Select", index);

	ret		= dsi6c500k_ioctl(fd, DSI6C500K_IOCTL_IRQ_SEL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DSI6C500K_IRQ_INIT_DONE:

			ptr	= "Initialize Done";
			break;

		case DSI6C500K_IRQ_AUTOCAL_DONE:

			ptr	= "Autocalibration Done";
			break;

		case DSI6C500K_IRQ_CHAN_READY:

			ptr	= "Channel Ready";
			break;

		case DSI6C500K_IRQ_AI_BUF_THR_L2H:

			ptr	= "AI Buffer Threshold Low-to-High";
			break;

		case DSI6C500K_IRQ_AI_BUF_THR_H2L:

			ptr	= "AI Buffer Threshold High-to-Low";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


